
from sqlobject import *

import time
import mx.DateTime


class BossConfig(dict):
    def __init__(self):
        import os
        import re

        dict.__init__(self)
        #self._dict = {}

        configFileName = "BossConfig.clad"
        bossTop = None
        homeDir = None
        possibleConfigFileNames = [configFileName]
        try:
            homeDir = os.environ['HOME']
            possibleConfigFileNames.append(os.path.join(homeDir,configFileName))
        except: pass
        try:
            bossTop = os.environ['BOSSDIR']
            possibleConfigFileNames.append(os.path.join(bossTop,configFileName))
        except: pass

        f = None
        for configFile in possibleConfigFileNames:
            try:
                f = open(configFile,'r')
            except: pass

        if not f:
            raise Exception("Cannot find BOSS configuration")

        lineNo = 0
        configPat = re.compile(r'^\s*(\w+)\s*=\s*("?[\w./*|;:!@$%=+-_]+"?)\s*;')
        for line in f.readlines():
            lineNo = lineNo + 1
            # Nuke comments
            commentStart = line.find('#')
            if commentStart >= 0:
                realLine = line[:commentStart]
                pass
            else:
                realLine = line
            m = configPat.match(realLine)
            if m:
                try:
                    self[m.group(1)] = eval(m.group(2))
                except:
                    print "Bad BOSSConfig line %d"%lineNo, line

        f.close()

        # Try to set up the database connection string thingee.
        try:
            host =  self['BOSS_DB_HOST']
            user = self['BOSS_DB_USER']
            passwd =  self['BOSS_DB_USER_PW']
            database = self['BOSS_DB_NAME']
            self['conn'] = "mysql://%s:%s@%s/%s" % (user, passwd, host, database)
        except:
            self['conn'] = None


try:
    config = BossConfig()
except:
    print "WARNING: No BOSS Config"
    config = None


def _timeToTicks(value):
    try:
        v = int(value)
    except:
        v = mx.DateTime.DateTimeFrom(value)
        v = int(v.ticks())
    return v


class BossScheduler (SQLObject):
    _table = "SCHEDULER"
    _idName = "name"
    _idType = str
    try:    _connection = config['conn']
    except: _connection = None
    name         = StringCol(length=30)
    default      = StringCol(length=1, dbName="def", default="N")
    top_work_dir = StringCol(length=100, default="NONE")
    submit_bin   = BLOBCol(default="")
    kill_bin     = BLOBCol(default="")
    query_bin    = BLOBCol(default="")


class BossJobType (SQLObject):
    _table = "JOBTYPE"
    _idName = "name"
    _idType = str
    try:    _connection = config['conn']
    except: _connection = None
    name         = StringCol(length=30)
    schema       = BLOBCol(default="")
    comment      = StringCol(length=100, default="")
    pre_bin      = BLOBCol(default="")
    run_bin      = BLOBCol(default="")
    post_bin     = BLOBCol(default="")


class BossJob (SQLObject):
    _table = "JOB"
    try:    _connection = config['conn']
    except: _connection = None
    type         = StringCol(length=30, default="")
    sch          = StringCol(length=30, default="")
    sid          = StringCol(length=100, default="")
    s_host       = StringCol(length=30, default="")
    s_path       = StringCol(length=255, default="")
    s_usr        = StringCol(length=30, default="")
    executable   = StringCol(length=255, dbName='exec', default="")
    args         = StringCol(length=255, default="")
    stdin        = StringCol(length=255, default="")
    stdout       = StringCol(length=255, default="")
    stderr       = StringCol(length=255, default="")
    e_host       = StringCol(length=30, default="")
    e_path       = StringCol(length=255, default="")
    e_usr        = StringCol(length=30, default="")
    log          = StringCol(length=255, default="")
    t_sub        = IntCol(default=0)
    t_start      = IntCol(default=0)
    t_stop       = IntCol(default=0)
    t_stat       = StringCol(length=50, default="")
    ret_code     = StringCol(length=11, default="")

    # BOSS is using raw unix seconds since epoch time idiocy
    # instead of native DB time types.  KILL ME NOW.
    #
    # Unhappily, Condor uses at least two different string
    # representations for time, so we can't do a cheap conversion
    # with python's native time module.  Probably a good thing as it
    # forces us to at least try to do time correctly.  A bad
    # thing because -- ARGH!!!

    # If we have mx (which we don't) we could use this nugget:
    #
    # >>> import mx.DateTime
    # >>> t= mx.DateTime.DateTimeFrom("11/09 14:28:35")
    # >>> print t
    # 2004-11-09 14:28:35.00
    # >>> t.ticks()
    # 1100032115.0


    def _set_t_sub(self, value):
        self._SO_set_t_sub(_timeToTicks(value))

    def _set_t_start(self, value):
        self._SO_set_t_start(_timeToTicks(value))

    def _set_t_stop(self, value):
        self._SO_set_t_stop(_timeToTicks(value))

    def stateString(self):
        """
        Return a human-readable string describing the state of the job.
        """
        if self.t_stat == "":
            return "wait"
        elif self.t_stat == "I":
            return "idle"
        elif self.t_stat == "R":
            return "run"
        elif self.t_stat == "A":
            return "abort"
        elif (self.t_stat == "T") and (self.ret_code == ""):
            return "stop"
        elif (self.t_stat == "T") and (self.ret_code == "0"):
            return "success"
        elif self.t_stat == "T":
            return "fail"
        else:
            raise ValueError, "unrecognized state"
